﻿using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Problems.Scheduling.MRCPSP {

  [Item(nameof(Mode), "")]
  [StorableClass]
  public sealed class Mode : ParameterizedNamedItem {
    //public Mode()
    //  : this() {
    //}

    [Storable]
    private readonly IFixedValueParameter<IntValue> durationParam;

    [Storable]
    private readonly IFixedValueParameter<IntValue> numberParam;

    [Storable]
    private readonly IValueParameter<ItemList<ResourceDemand>> resourceDemandsParam;

    public Mode() {
      Parameters.Add(numberParam = new FixedValueParameter<IntValue>(nameof(Number), new IntValue()));
      Parameters.Add(resourceDemandsParam = new ValueParameter<ItemList<ResourceDemand>>(nameof(ResourceDemands), new ItemList<ResourceDemand>()));
      Parameters.Add(durationParam = new FixedValueParameter<IntValue>(nameof(Duration), new IntValue()));
    }

    [StorableConstructor]
    private Mode(bool deserializing)
      : base(deserializing) { }

    private Mode(Mode original, Cloner cloner)
      : base(original, cloner) {
      numberParam = cloner.Clone(original.numberParam);
      resourceDemandsParam = cloner.Clone(original.resourceDemandsParam);
      durationParam = cloner.Clone(original.durationParam);
    }

    public int Duration {
      get => durationParam.Value.Value;
      set => durationParam.Value.Value = value;
    }

    public int Number {
      get => numberParam.Value.Value;
      set => numberParam.Value.Value = value;
    }

    public ItemList<ResourceDemand> ResourceDemands {
      get => resourceDemandsParam.Value;
      set => resourceDemandsParam.Value = value;
    }

    public override IDeepCloneable Clone(Cloner cloner) => new Mode(this, cloner);

    public override string ToString() => $"{Name} (Mode {Number})";
  }
}
